
require 'gitlab/email/handler'

module Gitlab
  module Email
    class Handler
      class CreateNote < Handler
        def can_handle?
          !!sent_notification
        end

        def execute
          raise SentNotificationNotFoundError unless sent_notification
          raise AutoGeneratedEmailError if mail.header.to_s =~ /auto-(generated|replied)/

          validate_permission!(:create_note)

          raise NoteableNotFoundError unless sent_notification.noteable
          raise EmptyEmailError if message.blank?

          verify_record(
            create_note,
            InvalidNoteError,
            "The comment could not be created for the following reasons:"
          )
        end

        def author
          sent_notification.recipient
        end

        def project
          sent_notification.project
        end

        def sent_notification
          @sent_notification ||= SentNotification.for(mail_key)
        end

        private
        def create_note
          Notes::CreateService.new(
            project,
            author,
            note:           message,
            noteable_type:  sent_notification.noteable_type,
            noteable_id:    sent_notification.noteable_id,
            commit_id:      sent_notification.commit_id,
            line_code:      sent_notification.line_code
          ).execute
        end
      end
    end
  end
end
